"""Define flunearyou constants."""
import logging

DOMAIN = "flunearyou"
LOGGER = logging.getLogger(__package__)

DATA_COORDINATOR = "coordinator"

CATEGORY_CDC_REPORT = "cdc_report"
CATEGORY_USER_REPORT = "user_report"
