"""Constants for the Template Platform Components."""

CONF_AVAILABILITY_TEMPLATE = "availability_template"

DOMAIN = "template"

PLATFORM_STORAGE_KEY = "template_platforms"

EVENT_TEMPLATE_RELOADED = "event_template_reloaded"

PLATFORMS = [
    "alarm_control_panel",
    "binary_sensor",
    "cover",
    "fan",
    "light",
    "lock",
    "sensor",
    "switch",
    "vacuum",
]
