"""Constants for the blueprint integration."""
BLUEPRINT_FOLDER = "blueprints"

CONF_BLUEPRINT = "blueprint"
CONF_USE_BLUEPRINT = "use_blueprint"
CONF_INPUT = "input"
CONF_SOURCE_URL = "source_url"

DOMAIN = "blueprint"
