"""Constants for the WiLight integration."""

DOMAIN = "wilight"

# Item types
ITEM_LIGHT = "light"

# Light types
LIGHT_ON_OFF = "light_on_off"
LIGHT_DIMMER = "light_dimmer"
LIGHT_COLOR = "light_rgb"

# Light service support
SUPPORT_NONE = 0
